﻿using UnityEngine;
using System.Collections;

[RequireComponent( typeof( SteamVR_TrackedObject ) )]
[RequireComponent( typeof( AudioSource ) )]
public class Pistol : MonoBehaviour
{
    public GameObject LaserPrefab;
    public float LaserLifetime;

    private SteamVR_TrackedObject trackedObject;
    private AudioSource laserAudioSource;

    private SteamVR_Controller.Device Controller
    {
        get
        {
            return SteamVR_Controller.Input( ( int )this.trackedObject.index );
        }
    }

    void Start()
    {
        this.trackedObject = this.GetComponentInParent<SteamVR_TrackedObject>();
        this.laserAudioSource = this.GetComponent<AudioSource>();
    }

    void Update()
    {
        if ( this.Controller.GetHairTriggerDown() )
        {
            // Shooting is what starts a new game if one isn't already running:
            if ( GameStateManager.Instance != null )
            {
                GameStateManager.Instance.StartNewGame();
            }

            // Provide auditory and haptic feedback:
            this.laserAudioSource.Play();
            this.StartCoroutine( this.ShootLaserHapticPulse() );

            // Create a visual effect:
            GameObject laser = GameObject.Instantiate<GameObject>( this.LaserPrefab );
            laser.transform.position = this.transform.position + 0.003204f * this.transform.up;
            laser.transform.forward = this.transform.forward;
            GameObject.Destroy( laser, this.LaserLifetime );

            // Look for a target, destroying it if we hit it:
            RaycastHit hit;
            if ( Physics.Raycast( this.transform.position, this.transform.forward, out hit ) )
            {
                // Make the laser stop at the thing it hit:
                laser.transform.localScale = new Vector3( 1, 1, hit.distance );

                // If we actually hit a target, destroy it:
                Target target = hit.collider.GetComponent<Target>();
                if ( target != null && target.DestroyedByPistol )
                {
                    target.PlayDestroyEffect( false );
                    GameObject.Destroy( target.gameObject );
                    GameStateManager.Instance.IncreaseScore();
                }
            }
            else
            {
                // Make the laser shoot off into the distance:
                laser.transform.localScale = new Vector3( 1, 1, 50 );
            }
        }
    }

    IEnumerator ShootLaserHapticPulse()
    {
        this.Controller.TriggerHapticPulse( 1000 );
        yield return new WaitForSeconds( 0.01f );
        this.Controller.TriggerHapticPulse( 1000 );
        yield return new WaitForSeconds( 0.01f );
        this.Controller.TriggerHapticPulse( 1000 );
        yield return new WaitForSeconds( 0.01f );
    }
}
