﻿using UnityEngine;
using System.Collections;

[RequireComponent( typeof( Rigidbody ) )]
[RequireComponent( typeof( MeshRenderer ) )]
public class Target : MonoBehaviour
{
    public Material SwordMaterial;
    public Material PistolMaterial;
    public GameObject SwordExplosion;
    public GameObject PistolExplosion;
    public float Velocity;

    internal bool DestroyedBySword { get; private set; }
    internal bool DestroyedByPistol { get; private set; }

    public void Initialize( Vector3 position, Vector3 direction, bool destroyedBySword )
    {
        Rigidbody rigidBody = this.GetComponent<Rigidbody>();
        this.transform.position = position;
        rigidBody.MovePosition( position );

        MeshRenderer meshRenderer = this.GetComponent<MeshRenderer>();
        if ( destroyedBySword )
        {
            this.DestroyedByPistol = false;
            this.DestroyedBySword = true;
            meshRenderer.material = this.SwordMaterial;
            rigidBody.velocity = direction * this.Velocity;
        }
        else
        {
            this.DestroyedByPistol = true;
            this.DestroyedBySword = false;
            meshRenderer.material = this.PistolMaterial;
            rigidBody.velocity = direction * this.Velocity * 0.7f + Vector3.up * this.Velocity * 0.3f;
        }
    }

    public void PlayDestroyEffect( bool destroyedBySword )
    {
        if ( destroyedBySword )
        {
            GameObject.Instantiate( this.SwordExplosion, this.transform.position, Quaternion.identity );
        }
        else
        {
            GameObject.Instantiate( this.PistolExplosion, this.transform.position, Quaternion.identity );
        }
    }
}
